﻿using System.Linq;
using System.Reflection;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.DependencyInjection;

namespace Breeze.Api
{
    public static class MvcBuilderExtensions
    {
		/// <summary>
		/// Finds all the types that are <see cref="Controller"/> and add them to the Api as services.
		/// </summary>
		/// <param name="builder">The builder</param>
		/// <param name="services">The services to look into</param>
		/// <returns>The Mvc builder</returns>
		public static IMvcBuilder AddControllers(this IMvcBuilder builder, IServiceCollection services)
		{			
			var controllerTypes = services.Where(s => s.ServiceType.GetTypeInfo().BaseType == typeof(Controller));
			foreach (var controllerType in controllerTypes)
			{				
				builder.AddApplicationPart(controllerType.ServiceType.GetTypeInfo().Assembly);
			}

			builder.AddControllersAsServices();
			return builder;
		}		
	}
}
